==> libgrinvin-invariants-java_1.0.2.orig.tar.gz: grinvin-invariants-1.0.2/org/grinvin/invariants/computers/chemical/indices/MeanDistanceMagnitude.java <==
/* MeanDistanceMagnitude.java
 * =========================================================================
 * This file is part of the GrInvIn project - http://www.grinvin.org
 * 
 * Copyright (C) 2005-2007 Universiteit Gent
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * A copy of the GNU General Public License can be found in the file
 * LICENSE.txt provided with the source distribution of this program (see
 * the META-INF directory in the source jar). This license can also be
 * found on the GNU website at http://www.gnu.org/licenses/gpl.html.
 * 
 * If you did not receive a copy of the GNU General Public License along
 * with this program, contact the lead developer, or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

package org.grinvin.invariants.computers.chemical.indices;

import org.grinvin.GraphBundleView;
import org.grinvin.invariants.RealValue;
import org.grinvin.invariants.computers.AbstractInvariantComputer;

/**
 * computes the mean information content on the distance magnitude of the graph.
 */
public class MeanDistanceMagnitude extends AbstractInvariantComputer {
    
    public RealValue compute(GraphBundleView bundle) {
        int[][] distanceMatrix = bundle.distanceMatrix();
        int n = distanceMatrix.length;
        if (n <= 1)
            return RealValue.undefinedValue(this);
        int max = distanceMatrix [1][0];
        double[] graphDistanceCount = new double[bundle.getGraph().getNumberOfEdges()+1];
        double wienerIndex = max;
        for (int i=2; i < n; i++) {
            for (int j=0; j < i; j++) {
                graphDistanceCount[distanceMatrix[i][j]]++;
                wienerIndex += distanceMatrix[i][j];
                if (distanceMatrix[i][j] == 0)
                    return RealValue.undefinedValue(this); // return undefined for disconnected graphs
                else if (distanceMatrix[i][j] > max)
                    max = distanceMatrix[i][j];
            }
        }
        double index = 0.0;
        for(int i=1; i<=max; i++)
            index -= graphDistanceCount[i]*i/wienerIndex*Math.log(i/wienerIndex)/Math.log(2);
        return new RealValue(index, this);
    }
    
    public String getInvariantId() {
        return "org.grinvin.invariants.MeanDistanceMagnitude";
    }
}

==> libgrinvin-invariants-java_1.2.orig.tar.gz: grinvin-invariants-1.2/org/grinvin/invariants/computers/chemical/indices/MeanDistanceMagnitude.java <==
/* MeanDistanceMagnitude.java
 * =========================================================================
 * This file is part of the GrInvIn project - http://www.grinvin.org
 * 
 * Copyright (C) 2005-2008 Universiteit Gent
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * A copy of the GNU General Public License can be found in the file
 * LICENSE.txt provided with the source distribution of this program (see
 * the META-INF directory in the source jar). This license can also be
 * found on the GNU website at http://www.gnu.org/licenses/gpl.html.
 * 
 * If you did not receive a copy of the GNU General Public License along
 * with this program, contact the lead developer, or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

package org.grinvin.invariants.computers.chemical.indices;

import org.grinvin.graphs.GraphBundleView;
import org.grinvin.invariants.computers.AbstractInvariantComputer;
import org.grinvin.invariants.values.RealValue;

/**
 * computes the mean information content on the distance magnitude of the graph.
 */
public class MeanDistanceMagnitude extends AbstractInvariantComputer {
    
    public RealValue compute(GraphBundleView bundle) {
        int[][] distanceMatrix = bundle.distanceMatrix();
        int n = distanceMatrix.length;
        if (n <= 1)
            return RealValue.undefinedValue(this);
        int max = distanceMatrix [1][0];
        double[] graphDistanceCount = new double[bundle.getGraph().getNumberOfEdges()+1];
        double wienerIndex = max;
        for (int i=2; i < n; i++) {
            for (int j=0; j < i; j++) {
                graphDistanceCount[distanceMatrix[i][j]]++;
                wienerIndex += distanceMatrix[i][j];
                if (distanceMatrix[i][j] == 0)
                    return RealValue.undefinedValue(this); // return undefined for disconnected graphs
                else if (distanceMatrix[i][j] > max)
                    max = distanceMatrix[i][j];
            }
        }
        double index = 0.0;
        for(int i=1; i<=max; i++)
            index -= graphDistanceCount[i]*i/wienerIndex*Math.log(i/wienerIndex)/Math.log(2);
        return new RealValue(index, this);
    }
    
    public String getInvariantId() {
        return "org.grinvin.invariants.MeanDistanceMagnitude";
    }
}
